UNIT GenMoves;

INTERFACE
USES
  Globals;


FUNCTION  GenMoveList(TTMove : Str4) : Byte;
FUNCTION  GenCaptureList : Byte;

FUNCTION  GetAMove(MaxMoves : Byte) : MoveRec;

PROCEDURE WriteMoveList(Index : Byte);   {-- for debugging }


IMPLEMENTATION
{-----------------------------------------------------------------------------------------------------------}
USES
  LookUpTables,
  Board,
  Utils,
  Search,
  Debugging;

TYPE
  EnemySet = SET OF Byte;

VAR
  MoveList      : ARRAY[0..MaxPly, 1..MaxMoves] OF MoveRec; {-- the generated moves for each ply }
  MoveListIndex : Byte;
  HashMove      : Str4;

{-- for debugging }
PROCEDURE WriteMoveList(Index : Byte);
VAR
  i : Byte;
  f : Text;
BEGIN
  Assign(f, 'MoveList.txt');
  ReWrite(f);

  WriteLn(f, 'Ply=', Ply);

  FOR i := 1 TO Index DO
    BEGIN
      Write(f, i, ' ',SqToStr[MoveList[Ply, i].FromSq]);
      IF MoveList[Ply, i].Capture THEN
        Write(f, 'x');
      Write(f, SqToStr[MoveList[Ply, i].ToSq]);
      WriteLn(f, '  ', MoveList[Ply, i].MoveValue);
    END;
  WriteLn(f, '-----------------------------------');
  Close(f);
END;


FUNCTION MoveValue(FromSq, ToSq : Int8; Piece : PieceType) : Int16;
{-- used for move ordering using PST + history tables }
BEGIN
  IF WhosMove = Bmove THEN  {-- flip the lookup tables for blacks point of view }
    BEGIN
      FromSq := FromSq XOR 56;
      ToSq   := ToSq   XOR 56;
    END;

  CASE Piece OF
    Wpawn,   Bpawn   : MoveValue := Pst_pawn  [ToSq] + HistTab[Piece, ToSq];
    wnight,  Bnight  : MoveValue := Pst_knight[ToSq] + HistTab[Piece, ToSq];
    Wbishop, Bbishop : MoveValue := Pst_bishop[ToSq] + HistTab[Piece, ToSq];
    Wrook,   Brook   : MoveValue := Pst_rook  [ToSq] + HistTab[Piece, ToSq];
    Wqueen,  Bqueen  : MoveValue := Pst_queen [ToSq] + HistTab[Piece, ToSq];
    Wking,   Bking   : IF NOT EndGame THEN
                         MoveValue := Pst_king_mg[ToSq] + HistTab[Piece, ToSq]
                       ELSE
                         MoveValue := Pst_king_eg[ToSq] + HistTab[Piece, ToSq];
  END
END;


PROCEDURE AddToMoveList(From_Sq, To_Sq, Value : Int16);
VAR
  Enemy    : SET OF Byte;
  IsCap    : Boolean;
  Piece,
  Victem,
  Attacker : PieceType;
BEGIN
  Inc(MoveListIndex);  {-- now points to next empty slot }

  IF WhosMove = Wmove THEN
    Enemy := [Bpawn..Bking]
  ELSE
    Enemy := [Wpawn..Wking];

  Piece := WhatPiece(From_Sq);

  IF SquaresToStr(From_Sq, To_Sq) = HashMove THEN
    Value := 1001;
(*
  IF (Killers[0, Ply].FromSq = From_Sq) AND (Killers[0, Ply].ToSq = To_Sq) THEN
    Value := 300;  {-- should be somewhere less than the good captures }
  IF (Killers[1, Ply].FromSq = From_Sq) AND (Killers[1, Ply].ToSq = To_Sq) THEN
    Value := 250;  {-- should be somewhere less than the good captures }
*)
  {-- Give a value in order of LVA/MVV }
  IsCap := (WhatPiece(To_Sq) IN Enemy) OR (To_Sq = Data[Ply].EnPasSq);
  IF IsCap THEN
    BEGIN
      Attacker := Piece AND %0111;             {-- converts black pieces to white }
      Victem   := WhatPiece(To_Sq) AND %0111;  {-- converts black pieces to white }

      IF (Data[Ply].EnPasSq <> To_Sq) THEN
        Value := Value + PieceValue[Victem] - Attacker   {-- queen range 892 -> 895 }
      ELSE                                               {-- rooks range 499 -> 495 }
        Value := Value + 99; {-- which is PxP }          {-- pawns range  99 ->  95 }
    END
  ELSE
    Value := Value;

  WITH MoveList[Ply, MoveListIndex] DO
    BEGIN
      FromSq    := From_Sq;
      ToSq      := To_Sq;
      MoveValue := Value;
      Castle    := (Piece IN [Wking, Bking]) AND (Abs(From_Sq - To_Sq) = 2);
      Capture   := IsCap;
      Promotion := ( (Piece = Wpawn) AND (To_Sq > 55) ) OR
                   ( (Piece = Bpawn) AND (To_Sq < 8) );
      IF Promotion THEN
        MoveValue := 1000;   {-- better score than any capture }
    END;
END;


FUNCTION OnBoard(Rank, Fyle : Int8) : Boolean; INLINE;
BEGIN
  OnBoard := (Rank IN [0..7]) AND (Fyle IN [0..7]);
END;


FUNCTION GenMoveList(TTMove : Str4) : Byte;
VAR
  PieceSq : Int8;


  PROCEDURE GenPawnMoves;
  CONST
    {-- 1 or 2 ranks up and 1 rank up and 1 file left and 1 file right }
    OffSet      : ARRAY[0..3, 0..1] OF Int8 = ( (1, 0), (2, 0), (1, -1), (1, 1) );
                       {rank, file}
    BoardOffSet : ARRAY[0..3] OF Int8 = (8, 16, 7, 9);  {-- board squares offset }
  VAR
    i,
    Direction,
    FromSq,
    ToSq,
    SqRank,
    SqFile,
    TargetRank,
    TargetFile : Int8;       {-- whats on this square }
    Piece      : Byte;


    FUNCTION IllegalPawnMove(P : PieceType) : Boolean;
    {-- P is piece on the To Square }
    BEGIN
      IllegalPawnMove := True;  {-- unless proven other wise }

      {-- 1 sq up }
      IF (Direction = 0) AND (P = Empty) THEN
        Exit(False);

      {-- 2 sq up }
      IF (Direction = 1) AND
         (NOT(FromSq IN [16..47])) AND
         (WhatPiece(ToSq - 8*WhosMove) = Empty) AND    {-- i is -1 for Black as it is coming down the board }
         (P = Empty) THEN
        Exit(False);

      {-- capture moves }
      IF (Direction > 1) THEN
        BEGIN
          IF ToSq = Data[Ply].EnPasSq THEN
            Exit(False);

          {-- normal capture }
          IF ( (WhosMove =  1) AND (P IN [Bpawn..Bqueen]) ) OR
             ( (WhosMove = -1) AND (P IN [Wpawn..Wqueen]) ) THEN
            Exit(False);
        END;
    END;


  BEGIN
    FOR i := 0 TO 7 DO   {-- get each pawn in turn }
      BEGIN
        IF WhosMove = Wmove THEN
          FromSq := Data[Ply].WpawnList[i]  {-- get a pawn }
        ELSE
          FromSq := Data[Ply].Bpawnlist[i];

        IF FromSq <> NoPiece THEN
          BEGIN
            {-- get the squares rank and file }
            SqRank := FromSq SHR 3;   {-- divide by 8 }
            SqFile := FromSq MOD 8;

            FOR Direction := 0 TO 3 DO  {-- a pawn has 4 possible moves }
              BEGIN
                TargetRank := SqRank + WhosMove*OffSet[Direction, 0];
                TargetFile := SqFile + WhosMove*OffSet[Direction, 1];
                ToSq       := FromSq + WhosMove*BoardOffSet[Direction];

                IF OnBoard(TargetRank, TargetFile) THEN
                  Piece := WhatPiece(ToSq)         {-- an enemy or a vacant square }
                ELSE
                  Continue;

                IF IllegalPawnMove(Piece) THEN
                  Continue;

                AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, Wpawn));  {-- doesnt matter if its Wpawn or Bpawn }
              END;  {-- for }
          END;    {-- if }
      END;      {-- for }
  END;


  PROCEDURE GenKnightMoves(FromSq : Int8);
  CONST
    {-- 8 posible offsets for the knight }
    OffSet : ARRAY[0..7, 0..1] OF Int8 = ( (1, -2), (1, 2), (-1, -2), (-1, 2),
                  {Rank, File}             (2, -1), (2, 1), (-2, -1), (-2, 1) );

    BoardOffSet : ARRAY[0..7] OF Int8  = (6, 10, -10, -6, 15, 17, -17, -15);
  VAR
    Direction : Int8;
    SqRank,
    SqFile,
    TargetRank,
    TargetFile,
    ToSq      : Int8;
    Piece     : Byte;
    MyPiece   : Boolean;

  BEGIN
    SqRank := FromSq SHR 3;      {-- divide by 8 }
    SqFile := FromSq MOD 8;

    FOR Direction := 0 TO 7 DO
      BEGIN
        TargetRank := SqRank + OffSet[Direction, 0];
        TargetFile := SqFile + OffSet[Direction, 1];

        IF OnBoard(TargetRank, TargetFile) THEN
          BEGIN
            ToSq := FromSq + BoardOffSet[Direction];
            Piece := WhatPiece(ToSq);
          END
        ELSE
          Continue;         {-- next direction }

        IF WhosMove = Wmove THEN
          MyPiece := Piece IN [Wpawn..Wking]
        ELSE
          MyPiece := Piece IN [Bpawn..Bking];

        IF MyPiece THEN  {-- landed on a square of your own pieces, so get next direction }
          Continue;

        AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, Bnight));
      END;
  END;


  PROCEDURE GenRookMoves(FromSq : Int8; IsQueen : Boolean);
  CONST
    OffSet : ARRAY[0..3, 0..1] OF Int8 = ( (0, 1), (0, -1), (1, 0), (-1, 0) );
  VAR
    Direction : Int8;
    SqRank,
    SqFile,
    TargetRank,
    TargetFile,
    ToSq      : Int8;       {-- whats on this square }
    Piece     : Byte;
    MyPiece,
    OffBoard,
    Capture   : Boolean;

  BEGIN
    FOR Direction := 0 TO 3 DO
      BEGIN
        {-- remember the original square }
        SqRank := FromSq SHR 3;      {-- divide by 8 }
        SqFile := FromSq MOD 8;

        REPEAT {-- each square in a direction }
          TargetRank := SqRank + OffSet[Direction, 0];
          TargetFile := SqFile + OffSet[Direction, 1];

          IF OnBoard(TargetRank, TargetFile) THEN
            BEGIN
              ToSq     := (TargetRank SHL 3) + TargetFile;
              Piece    := WhatPiece(ToSq);
              OffBoard := False;
            END
          ELSE
            BEGIN
              OffBoard := True;
              Continue;         {-- next direction }
            END;

          IF WhosMove = Wmove THEN
            BEGIN
              MyPiece := Piece IN [Wpawn..Wking];
              Capture := Piece IN [Bpawn..Bking];
            END
          ELSE
            BEGIN
              MyPiece := Piece IN [Bpawn..Bking];
              Capture := Piece IN [Wpawn..Wking];
            END;

          IF MyPiece THEN
            Continue;

          IF IsQueen THEN
            AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, Wqueen))
          ELSE
            AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, Wrook));

          IF Capture THEN   {-- has added the capture, so get the next direction }
            Continue;

          {-- go to next square }
          SqRank := TargetRank;
          SqFile := TargetFile;
        UNTIL OffBoard OR MyPiece OR Capture;
      END;
  END;


  PROCEDURE GenBishopMoves(FromSq : Int8; IsQueen : Boolean);
  CONST
    OffSet : ARRAY[0..3, 0..1] OF Int8 = ( (1, 1), (-1, -1), (1, -1), (-1, 1) );
  VAR
    Direction : Int8;
    SqRank,
    SqFile,
    TargetRank,
    TargetFile,
    ToSq      : Int8;       {-- whats on this square }
    Piece     : Byte;
    MyPiece,
    Capture,
    OffBoard  : Boolean;

  BEGIN
    FOR Direction := 0 TO 3 DO
      BEGIN
        {-- remember the original square }
        SqRank := FromSq SHR 3;      {-- divide by 8 }
        SqFile := FromSq MOD 8;

        REPEAT
          TargetRank := SqRank + OffSet[Direction, 0];
          TargetFile := SqFile + OffSet[Direction, 1];

          IF OnBoard(TargetRank, TargetFile) THEN
            BEGIN
              ToSq     := (TargetRank SHL 3) + TargetFile;
              Piece    := WhatPiece(ToSq);
              OffBoard := False;
            END
          ELSE
            BEGIN
              OffBoard := True;
              Continue;         {-- next direction }
            END;

          IF WhosMove = Wmove THEN
            BEGIN
              MyPiece := Piece IN [Wpawn..Wking];
              Capture := Piece IN [Bpawn..Bking];
            END
          ELSE
            BEGIN
              MyPiece := Piece IN [Bpawn..Bking];
              Capture := Piece IN [Wpawn..Wking];
            END;

          IF MyPiece THEN
            Continue;

          IF IsQueen THEN
            AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, Wqueen))
          ELSE
            AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, Wbishop));

          IF Capture THEN
            Continue;

          {-- go to next square }
          SqRank := TargetRank;
          SqFile := TargetFile;
        UNTIL OffBoard OR MyPiece OR Capture;
      END;
  END;


  PROCEDURE GenQueenMoves(FromSq : Int8);
  BEGIN
    GenBishopMoves(FromSq, True);
    GenRookMoves  (FromSq, True);
  END;


  PROCEDURE GenKingMoves(FromSq : Int8);
  CONST
    {-- 8 posible offsets for the king }
    OffSet : ARRAY[0..7, 0..1] OF Int8 = ( (1, 0), (1, 1), (0, 1), (-1, 1),
                   {Rank, File}            (-1, 0), (-1, -1), (0, -1), (1, -1) );

    BoardOffSet : ARRAY[0..7] OF Int8  = (8, 9, 1, -7, -8, -9, -1, 7);
  VAR
    Direction : Int8;
    SqRank,
    SqFile,
    TargetRank,
    TargetFile,
    ToSq      : Int8;       {-- whats on this square }
    Piece     : Byte;
    MyPiece   : Boolean;

  BEGIN
    SqRank := FromSq SHR 3;      {-- divide by 8 }
    SqFile := FromSq MOD 8;

    FOR Direction := 0 TO 7 DO
      BEGIN
        TargetRank := SqRank + OffSet[Direction, 0];
        TargetFile := SqFile + OffSet[Direction, 1];

        IF OnBoard(TargetRank, TargetFile) THEN
          BEGIN
            ToSq := FromSq + BoardOffSet[Direction];
            Piece := WhatPiece(ToSq);
          END
        ELSE
          Continue;         {-- next direction }

        IF WhosMove = Wmove THEN
          MyPiece := Piece IN [Wpawn..Wking]
        ELSE
          MyPiece := Piece IN [Bpawn..Bking];

        IF MyPiece OR (WhatPiece(ToSq) in [Wking, Bking]) THEN  {-- landed on a square of your own pieces, so get next direction }
          Continue;

        AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, Bking));
      END;

    {-- Castling moves......very messy }
    IF (WhosMove = Wmove) THEN
      BEGIN
        IF FromSq = 4 THEN  {-- white king is on home square }
          BEGIN
            IF (Data[Ply].CastleStat AND %00000001) = 1 THEN             {-- allowed to castling short }
              IF ((Data[Ply].Board[0] SHR 20) AND %11111111) = 0 THEN  {-- squares 5 and 6 are empty }
                 IF NOT(Attacked(4, Bmove)) AND NOT(Attacked(5, Bmove)) AND NOT(Attacked(6, Bmove)) THEN
                   AddToMoveList(FromSq, 6, 150);

            IF Data[Ply].CastleStat AND %00000010 = %00000010 THEN          {-- allowed to castle long castle long }
              IF ((Data[Ply].Board[0] SHR 4) AND %111111111111) = 0 THEN  {-- squares 1, 2, 3 are empty }
                IF NOT(Attacked(4, Bmove)) AND NOT(Attacked(2, Bmove)) AND NOT(Attacked(3, Bmove)) THEN
                  AddToMoveList(FromSq, 2, 150);
          END;
      END;

    IF (WhosMove = Bmove) THEN
      BEGIN
        IF FromSq = 60 THEN
          BEGIN
            IF Data[Ply].CastleStat AND %00000100 = %00000100 THEN        {-- allowed to castle short }
              IF ((Data[Ply].Board[7] SHR 20) AND %11111111) = 0 THEN   {-- squares 61 and 62 are empty }
                 IF NOT(Attacked(60, Wmove)) AND NOT(Attacked(61, Wmove)) AND NOT(Attacked(62, Wmove)) THEN
                   AddToMoveList(FromSq, 62, 150);

            IF Data[Ply].CastleStat AND %00001000 = %00001000 THEN          {-- allowed to castle long }
              IF ((Data[Ply].Board[7] SHR 4) AND %111111111111) = 0 THEN  {-- squares 5 and 6 are empty }
                IF NOT(Attacked(60, Wmove)) AND NOT(Attacked(58, Wmove)) AND NOT(Attacked(59, Wmove)) THEN
                  AddToMoveList(FromSq, 58, 150);
          END;
      END;
  END;


{---------------------------- GenMoveList --------------------------------------}
BEGIN
  FillChar(MoveList[Ply], SizeOf(MoveList[Ply]), #0);
  MoveListIndex := 0;
  HashMove := TTmove;

  WITH Data[Ply] DO
    IF WhosMove = Wmove THEN
      BEGIN
        {--Wpawns }
        GenPawnMoves;

        {-- The two Knights }
        PieceSq := WpieceList[0, Wnight];
        IF PieceSq <> NoPiece THEN
          GenKnightMoves(PieceSq);
        PieceSq := WpieceList[1, Wnight];
        IF PieceSq <> NoPiece THEN
          GenKnightMoves(PieceSq);

        {-- The two Rooks }
        PieceSq := WpieceList[0, Wrook];
        IF PieceSq <> NoPiece THEN
          GenRookMoves(PieceSq, False);
        PieceSq := WpieceList[1, Wrook];
        IF PieceSq <> NoPiece THEN
          GenRookMoves(PieceSq, False);

        {-- The two Bishops }
        PieceSq := WpieceList[0, Wbishop];
        IF PieceSq <> NoPiece THEN
          GenBishopMoves(PieceSq, False);
        PieceSq := WpieceList[1, Wbishop];
        IF PieceSq <> NoPiece THEN
          GenBishopMoves(PieceSq, False);

        {-- The two Queen }
        PieceSq := WpieceList[0, Wqueen];
        IF PieceSq <> NoPiece THEN
          GenQueenMoves(PieceSq);
        PieceSq := WpieceList[1, Wqueen];
        IF PieceSq <> NoPiece THEN
          GenQueenMoves(PieceSq);

        {-- King }
        PieceSq := WpieceList[0, Wking];
        IF PieceSq <> -1 THEN    {-- should not really happen }
          GenKingMoves(PieceSq);
      END
    ELSE
      BEGIN
        {-- Bpawns }
        GenPawnMoves;

        {-- The two Knights }
        PieceSq := BpieceList[0, Bnight];
        IF PieceSq <> NoPiece THEN
          GenKnightMoves(PieceSq);
        PieceSq := BpieceList[1, Bnight];
        IF PieceSq <> NoPiece THEN
          GenKnightMoves(PieceSq);

        {-- The two Rooks }
        PieceSq := BpieceList[0, Brook];
        IF PieceSq <> NoPiece THEN
          GenRookMoves(PieceSq, False);
        PieceSq := BpieceList[1, Brook];
        IF PieceSq <> NoPiece THEN
          GenRookMoves(PieceSq, False);

        {-- The two Bishops }
        PieceSq := BpieceList[0, Bbishop];
        IF PieceSq <> NoPiece THEN
          GenBishopMoves(PieceSq, False);
        PieceSq := BpieceList[1, Bbishop];
        IF PieceSq <> NoPiece THEN
          GenBishopMoves(PieceSq, False);

        {-- Queen }
        PieceSq := BpieceList[0, Bqueen];
        IF PieceSq <> NoPiece THEN
          GenQueenMoves(PieceSq);
        PieceSq := BpieceList[1, Bqueen];
        IF PieceSq <> NoPiece THEN
          GenQueenMoves(PieceSq);

        {-- King }
        PieceSq := BpieceList[0, Bking];
        IF PieceSq <> -1 THEN
          GenKingMoves(PieceSq);
      END;
  GenMoveList := MoveListIndex;  {-- equals the number of moves }
{if ply = 10 then begin writemovelist(movelistindex); halt; end;}
END;  {-- GenMoveList }




FUNCTION GenCaptureList : Byte;
VAR
  i,
  Sq  : Int8;



  PROCEDURE GenWhitePawnCaps;
  VAR
    i      : Byte;
    FromSq,
    ToSq   : Int8;
  BEGIN
    FOR i := 0 TO 7 DO
      BEGIN
        FromSq := Data[Ply].WpawnList[i];

        IF FromSq <> -1 THEN
          BEGIN
            {-- left captures }
            ToSq := FromSq + 7;
            IF (WhatFile(FromSq) IN [1..7]) AND (WhatPiece(ToSq) IN [Bpawn..Bqueen]) THEN
              AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, Wpawn));

            {-- right captures }
            ToSq := FromSq + 9;
            IF (WhatFile(FromSq) IN [0..6]) AND (WhatPiece(ToSq) IN [Bpawn..Bqueen]) THEN
              AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, Wpawn));

          END;  {-- if }
      END; {-- for }
  END;


  PROCEDURE GenBlackPawnCaps;
  VAR
    i      : Byte;
    FromSq,
    ToSq   : Int8;
  BEGIN
    FOR i := 0 TO 7 DO
      BEGIN
        FromSq := Data[Ply].BpawnList[i];

        IF FromSq <> -1 THEN
          BEGIN
            {-- left captures }
            ToSq := FromSq - 9;
            IF (WhatFile(FromSq) IN [1..7]) AND (WhatPiece(ToSq) IN [Wpawn..Wqueen]) THEN
              AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, Wpawn));

            {-- right captures }
            ToSq := FromSq - 7;
            IF (WhatFile(FromSq) IN [0..6]) AND (WhatPiece(ToSq) IN [Wpawn..Wqueen]) THEN
              AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, Wpawn));
          END;  {-- if }
      END; {-- for }
  END;


  PROCEDURE GenKnightCap(FromSq : Int8; Enemy : EnemySet);
  CONST
    {-- 8 posible offsets for the knight }
    OffSet : ARRAY[0..7, 0..1] OF Int8 = ( (1, -2), (1, 2), (-1, -2), (-1, 2),
                  {Rank, File}             (2, -1), (2, 1), (-2, -1), (-2, 1) );
  VAR
    ToSq,
    Dir,
    SqRank,
    SqFile,
    TargetRank,
    TargetFile : Int8;
  BEGIN
    SqRank := FromSq SHR 3;
    SqFile := FromSq MOD 8;

    FOR Dir := 0 TO 7 DO
      BEGIN
        TargetRank := SqRank + OffSet[Dir, 0];
        TargetFile := SqFile + OffSet[Dir, 1];
        ToSq       := (TargetRank SHL 3) + TargetFile;

        IF OnBoard(TargetRank, TargetFile) AND (WhatPiece(ToSq) IN Enemy)THEN
          AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, Wnight));
      END;
  END;


  PROCEDURE GenBishopCaps(FromSq : Int8; Enemy : EnemySet; MyPiece : PieceType);
  CONST
    OffSet : ARRAY[0..3, 0..1] OF Int8 = ( (1, 1), (-1, -1), (1, -1), (-1, 1) );
  VAR
    Dir,
    SqRank,
    SqFile,
    ToSq      : Int8;
  BEGIN
    FOR Dir := 0 TO 3 DO
      BEGIN
        {-- the first square to try }
        SqRank := (FromSq SHR 3) + OffSet[Dir, 0];
        SqFile := (FromSq MOD 8) + OffSet[Dir, 1];
        ToSq   := (SqRank SHL 3) + SqFile;

        WHILE OnBoard(SqRank, SqFile) AND (WhatPiece(ToSq) = Empty) DO
          BEGIN
            {-- go to next square }
            SqRank := SqRank + OffSet[Dir, 0];
            SqFile := SqFile + OffSet[Dir, 1];
            ToSq   := (SqRank SHL 3) + SqFile;
          END;

        IF OnBoard(SqRank, SqFile) AND (WhatPiece(ToSq) IN Enemy) THEN
          AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, MyPiece));
      END;  {-- for }
  END;


  PROCEDURE GenRookCaps(FromSq : Int8; Enemy : EnemySet; MyPiece : PieceType);
  CONST
    OffSet : ARRAY[0..3, 0..1] OF Int8 = ( (0, 1), (0, -1), (1, 0), (-1, 0) );
  VAR
    Dir,
    SqRank,
    SqFile,
    ToSq      : Int8;
  BEGIN
    FOR Dir := 0 TO 3 DO
      BEGIN
        {-- the first square to try }
        SqRank := (FromSq SHR 3) + OffSet[Dir, 0];
        SqFile := (FromSq MOD 8) + OffSet[Dir, 1];
        ToSq   := (SqRank SHL 3) + SqFile;

        WHILE OnBoard(SqRank, SqFile) AND (WhatPiece(ToSq) = Empty) DO
          BEGIN
            {-- go to next square }
            SqRank := SqRank + OffSet[Dir, 0];
            SqFile := SqFile + OffSet[Dir, 1];
            ToSq   := (SqRank SHL 3) + SqFile;
          END;

        IF OnBoard(SqRank, SqFile) AND (WhatPiece(ToSq) IN Enemy) THEN
          AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, MyPiece));
      END;
  END;


  PROCEDURE GenQueenCaps(FromSq : Int8; Enemy : EnemySet);
  BEGIN
    GenBishopCaps(FromSq, Enemy, Wqueen);   {-- doesnt matter if its a white or black queen }
    GenRookCaps(FromSq, Enemy, Wqueen);
  END;


  PROCEDURE GenKingCaps(FromSq : Int8; Enemy : EnemySet);
  CONST
    {-- 8 posible offsets for the king }
    OffSet : ARRAY[0..7, 0..1] OF Int8 = ( (1, 0), (1, 1), (0, 1), (-1, 1),
                   {Rank, File}            (-1, 0), (-1, -1), (0, -1), (1, -1) );

  VAR
    Dir,
    ToSq,
    Rank,
    Fyle : Int8;
  BEGIN
    FOR Dir := 0 TO 7 DO
      BEGIN
        Rank := (FromSq SHR 3) + OffSet[Dir, 0];
        Fyle := (FromSq MOD 8) + OffSet[Dir, 1];

        IF OnBoard(Rank, Fyle) THEN
          BEGIN
            ToSq := (Rank SHL 3) + Fyle;
            IF (WhatPiece(ToSq) IN Enemy) THEN
              AddToMoveList(FromSq, ToSq, MoveValue(FromSq, ToSq, Wking));
          END;
      END; {-- for }
  END;


{-------------------------------------------- GenCapList ----------------------------------}
BEGIN
  FillChar(MoveList[Ply], SizeOf(MoveList[Ply]), #0);
  MoveListIndex := 0;

  IF WhosMove = 1 THEN
    BEGIN
      GenWhitePawnCaps;

      {-- pieces }
      FOR i := 0 TO 1 DO
        BEGIN
          {-- knight captures }
          Sq := Data[Ply].WpieceList[i, Wnight];
          IF Sq <> -1 THEN
            GenKnightCap(Sq, [9..13]);

          {-- bishop captures }
          Sq := Data[Ply].WpieceList[i, Wbishop];
          IF Sq <> -1 THEN
            GenBishopCaps(Sq, [9..13], Wbishop);

          {-- rook captures }
          Sq := Data[Ply].WpieceList[i, Wrook];
          IF Sq <> -1 THEN
            GenRookCaps(Sq, [9..13], Wrook);

          {-- queen captures }
          Sq := Data[Ply].WpieceList[i, Wqueen];
          IF Sq <> -1 THEN
            GenQueenCaps(Sq, [9..13]);

          {-- king captures }
          IF i = 0 THEN
            BEGIN
              Sq := Data[Ply].WpieceList[0, Wking];
              IF Sq <> -1 THEN
                GenKingCaps(Sq, [9..13]);
            END;
        END;
    END {-- if }
  ELSE  {-- blacks moves }
    BEGIN
      GenBlackPawnCaps;

      {-- pieces }
      FOR i := 0 TO 1 DO
        BEGIN
          {-- knight captures }
          Sq := Data[Ply].BpieceList[i, Bnight];
          IF Sq <> -1 THEN
            GenKnightCap(Sq, [1..5]);

          {-- bishop captures }
          Sq := Data[Ply].BpieceList[i, Bbishop];
          IF Sq <> -1 THEN
            GenBishopCaps(Sq, [1..5], Bbishop);

          {-- rook captures }
          Sq := Data[Ply].BpieceList[i, Brook];
          IF Sq <> -1 THEN
            GenRookCaps(Sq, [1..5], Brook);

          {-- queen captures }
          Sq := Data[Ply].BpieceList[i, Bqueen];
          IF Sq <> -1 THEN
            GenQueenCaps(Sq, [1..5]);

          {-- king captures }
          IF i = 0 THEN
            BEGIN
              Sq := Data[Ply].BpieceList[0, Bking];
              IF Sq <> -1 THEN
                GenKingCaps(Sq, [1..5]);
            END;
        END;
    END; {-- else }

  GenCaptureList := MoveListIndex;
END;  {-- GenCapList }


FUNCTION  GetAMove(MaxMoves : Byte) : MoveRec;
{-- scans the movelist and picks the highest scorer }
CONST
  Sentinal = -32000; {-- flags the move has been picked already }
VAR
  i, j,
  Value,
  MaxScore : Int16;

BEGIN
  GetAMove := NoMove;  {-- unless proven otherwise }
  MaxScore := Sentinal;

  j := 1;
  i := 1;
  FOR i := 1 TO MaxMoves DO  {-- this would be a null at the end of the list }
    BEGIN
      Value := MoveList[Ply, i].MoveValue;
      IF Value > MaxScore THEN
        BEGIN
          MaxScore := Value;                {-- note highest score }
          j := i;                           {-- and save position in the array }
        END;
    END;

  IF MaxScore > Sentinal THEN
    GetAMove := MoveList[Ply, j];           {-- return highest scoring move }

  MoveList[Ply, j].MoveValue := Sentinal;   {-- flag to say move is used }
END;


BEGIN
END.
